/*
  Bear Engine

  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file model.tpp
 * \brief Implementation of the bear::engine::model class.
 * \author Julien Jorge
 */
#include "engine/level_globals.hpp"

#include <set>

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 */
template<class Base>
bear::engine::model<Base>::model()
  : m_action(NULL), m_snapshot(NULL), m_sample(NULL)
{

} // model::model()

/*----------------------------------------------------------------------------*/
/**
 * \brief Copy constructor.
 * \param that The instance to copy from.
 */
template<class Base>
bear::engine::model<Base>::model( const model<Base>& that )
  : super(that), m_actor(that.m_actor), m_action(NULL),
    m_snapshot(NULL), m_sample(NULL)
{

} // model::model()

/*----------------------------------------------------------------------------*/
/**
 * \brief Destructor
 */
template<class Base>
bear::engine::model<Base>::~model()
{
  clear();
} // model::~model()

/*----------------------------------------------------------------------------*/
/**
 * \brief Assignment operator.
 * \param that The instance to copy from.
 */
template<class Base>
bear::engine::model<Base>&
bear::engine::model<Base>::operator=( const model<Base>& that )
{
  model<Base> tmp(that);
  std::swap(tmp, *this);
  return *this;
} // model::operator=()

/*----------------------------------------------------------------------------*/
/**
 * \brief Do the progression of the model.
 * \param elapsed_time Elapsed time since the last call.
 */
template<class Base>
void bear::engine::model<Base>::progress( universe::time_type elapsed_time )
{
  progress_model(elapsed_time);
} // model::progress()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the sprite representing the item.
 * \param visuals (out) The sprites of the item, and their positions.
 */
template<class Base>
void
bear::engine::model<Base>::get_visual( std::list<scene_visual>& visuals ) const
{
  if ( (m_snapshot!=NULL) && (m_action!=NULL) )
    {
      model_snapshot::const_mark_placement_iterator it;

      for ( it=m_snapshot->mark_placement_begin();
            it!=m_snapshot->mark_placement_end(); ++it )
        {
          const model_mark& mark( m_action->get_mark( it->get_mark_id() ) );

          if ( mark.has_animation() && it->is_visible() )
            {
              scene_visual v = this->get_scene_visual( *mark.get_animation() );
              model_mark_placement m(*it);

              get_oriented_mark_placement(m);

              if ( mark.apply_angle_to_animation() )
                {
                  double a
                    ( v.scene_element.get_rendering_attributes().get_angle()
                      + m.get_angle() );

                  if ( this->get_rendering_attributes().is_mirrored() )
                    a -= 3.14159;

                  v.scene_element.get_rendering_attributes().set_angle(a);
                }

              v.z_position += m.get_depth_position();

              v.scene_element.get_position() =
                m.get_position() + this->get_bottom_left();
              v.scene_element.get_position().x -=
                v.scene_element.get_bounding_box().width / 2;
              v.scene_element.get_position().y +=
                v.scene_element.get_bounding_box().height / 2;

              if ( this->get_rendering_attributes().is_mirrored() )
                v.scene_element.get_position().x -= this->get_gap().x;
              else
                v.scene_element.get_position().x += this->get_gap().x;

              if ( this->get_rendering_attributes().is_flipped() )
                v.scene_element.get_position().y -= this->get_gap().y;
              else
                v.scene_element.get_position().y += this->get_gap().y;

              visuals.push_front(v);
            }
        }
    }
} // model::get_visual()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the model actor.
 * \param actor The model_actor to copy from.
 */
template<class Base>
void bear::engine::model<Base>::set_model_actor( const model_actor& actor )
{
  clear();
  m_actor = actor;
} // model::set_model_actor()

/*----------------------------------------------------------------------------*/
/**
 * \brief Start an action.
 * \param name The name of the action to start.
 */
template<class Base>
void bear::engine::model<Base>::start_model_action( const std::string& name )
{
  const model_action* a = m_actor.get_action( name );

  if ( a != NULL )
    {
      if ( a == m_action )
        reset_action(0);
      else
        {
          stop_action();
          m_action = a;
          m_action_name = name;
          start_action(0);
        }

      execute_snapshot();
    }
} // model::start_model_action()

/*----------------------------------------------------------------------------*/
/**
 * \brief Switch to an other action.
 * \param name The name of the action to start.
 * \remark The difference with start_model_action is that the date is not reset
 *         to zero. The new action begins at the date of the current action.
 */
template<class Base>
void
bear::engine::model<Base>::switch_to_model_action( const std::string& name )
{
  const model_action* a = m_actor.get_action( name );

  if ( a != NULL )
    if ( a != m_action )
      {
        const universe::time_type d(m_date);

        stop_action();
        m_action = a;
        m_action_name = name;
        start_action(d);
      }
} // model::switch_to_model_action()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the name of the current action.
 */
template<class Base>
const std::string& bear::engine::model<Base>::get_current_action_name() const
{
  return m_action_name;
} // model::get_current_action_name()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get an action from the model actor.
 */
template<class Base>
bear::engine::model_action const*
bear::engine::model<Base>::get_action( const std::string& name ) const
{
  return m_actor.get_action(name);
} // model::get_action()

/*----------------------------------------------------------------------------*/
/**
 * \brief Execute a function associated with a snapshot.
 * \param name The name of the function.
 */
template<class Base>
void bear::engine::model<Base>::execute_function( const std::string& name )
{
  // nothing to do
} // model::execute_function()

/*----------------------------------------------------------------------------*/
/**
 * \brief Do the progression of the model.
 * \param elapsed_time Elapsed time since the last call.
 */
template<class Base>
void
bear::engine::model<Base>::progress_model( universe::time_type elapsed_time )
{
  if ( (m_snapshot!=NULL) && (m_action!=NULL) )
    {
      m_date += elapsed_time;

      if ( m_date <= m_action->get_duration() )
        {
          const model_snapshot* s = m_action->get_snapshot_at(m_date);

          if ( s != m_snapshot )
            {
              m_snapshot = s;
              execute_snapshot();
            }

          progress_animations(elapsed_time);
          update_sound_position();
        }
      else
        {
          const std::string action_name( m_action->get_next_action() );
          const universe::time_type t( m_date - m_action->get_duration() );

          if ( action_name.empty() )
            {
              progress_animations(elapsed_time);
              update_sound_position();
            }
          else
            {
              start_model_action( m_action->get_next_action() );
              progress_model( t );
            }
        }
    }
} // model::progress_model()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set a substitute for the animation of all marks with a given name.
 * \param mark_name The name of the marks.
 * \param anim The new animation on the marks.
 */
template<class Base>
void bear::engine::model<Base>::set_global_substitute
( const std::string& mark_name, const model_animation& anim )
{
  m_actor.set_global_substitute(mark_name, anim);
} // model::set_global_substitute()

/*----------------------------------------------------------------------------*/
/**
 * \brief Restore the default animation of all marks with a given name.
 * \param mark_name The name of the marks.
 */
template<class Base>
void bear::engine::model<Base>::remove_global_substitute
( const std::string& mark_name )
{
  m_actor.remove_global_substitute(mark_name);
} // model::remove_global_substitute()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the identifier of a mark, given its name.
 * \param mark_name The name of the mark.
 */
template<class Base>
std::size_t bear::engine::model<Base>::get_mark_id
( const std::string& mark_name ) const
{
  if ( m_action != NULL )
    return m_action->get_mark_id(mark_name);
  else
    return model_action::not_an_id;
} // model::get_mark_id()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the data associated to a mark placement, relatively to the world.
 * \param mark_name The name of the mark.
 * \param m (out) The mark placement.
 */
template<class Base>
bool bear::engine::model<Base>::get_mark_placement
( const std::string& mark_name, model_mark_placement& m ) const
{
  if ( m_action != NULL )
    return get_mark_placement( get_mark_id(mark_name), m );
  else
    return false;
} // model::get_mark_placement()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the data associated to a mark placement, relatively to the world.
 * \param i The id of the mark.
 * \param m (out) The mark placement.
 */
template<class Base>
bool bear::engine::model<Base>::get_mark_placement
( std::size_t i, model_mark_placement& m ) const
{
  bool result = false;
  if ( m_action != NULL )
    {
      if ( i < m_action->get_marks_count() )
        {
          m.set_mark_id( i );
          result = get_mark_placement(m);
        }
    }

  return result;
} // model::get_mark_placement()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the data associated to a mark placement, relatively to the world.
 * \param m (in/out) The mark placement.
 *
 * \remark You must set the mark identifier of \a m before calling.
 */
template<class Base>
bool
bear::engine::model<Base>::get_mark_placement( model_mark_placement& m ) const
{
  bool result(false);

  if ( m_snapshot != NULL )
    {
      result = get_oriented_mark_placement(m);

      m.set_position( m.get_position() + this->get_bottom_left() );
      m.set_angle( m.get_angle() + this->get_visual_angle() );
      m.set_depth_position( m.get_depth_position() + this->get_z_position() );
    }

  return result;
} // model::get_mark_placement()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the data associated to a mark placement, relatively to the
 *        orientation of the model.
 * \param m (in/out) The mark placement.
 *
 * \remark You must set the mark identifier of \a m before calling.
 */
template<class Base>
bool
bear::engine::model<Base>::get_oriented_mark_placement
( model_mark_placement& m ) const
{
  bool result(false);
  if ( m_snapshot != NULL )
    if ( m.get_mark_id() < m_action->get_marks_count() )
      {
        result = true;
        m = m_snapshot->get_mark_placement( m.get_mark_id() );

        universe::coordinate_type dx( m.get_position().x );
        universe::coordinate_type dy( m.get_position().y );
        double a( m.get_angle() );

        const universe::coordinate_type d = std::sqrt( dx * dx + dy * dy );
        const double pos_a = std::atan2(dy, dx);

        dx = d * std::cos(pos_a);
        dy = d * std::sin(pos_a);

        if ( this->get_rendering_attributes().is_mirrored() )
          {
            dx = this->get_width() - dx;
            a = 3.14159 - a;
          }

        if ( this->get_rendering_attributes().is_flipped() )
          {
            dy = this->get_height() - dy;
            a = -a;
          }

        m.set_position( dx, dy );
        m.set_angle(a);
      }

  return result;
} // model::get_oriented_mark_placement()

/*----------------------------------------------------------------------------*/
/**
 * \brief Start the current action.
 * \param d The date at which the action is reset (new start).
 */
template<class Base>
void bear::engine::model<Base>::start_action( universe::time_type d )
{
  CLAW_PRECOND(m_action != NULL);

  delete m_sample;
  m_sample = NULL;

  const std::string s( m_action->get_sound_name() );
  
  if ( this->get_level_globals().sound_exists(s) )
    {
      m_sample = this->get_level_globals().new_sample(s);
      audio::sound_effect effect;
      if ( !m_action->sound_is_global() )
        effect.set_position( this->get_center_of_mass() );
      effect.set_loops(0);
      m_sample->play( effect );
      //m_sample->set_position(d);
    }
  
  reset_action(d);
} // model::start_action()

/*----------------------------------------------------------------------------*/
/**
 * \brief Stop the current action.
 */
template<class Base>
void bear::engine::model<Base>::stop_action()
{
  if ( m_sample!=NULL )
    {
      m_sample->stop();
      delete m_sample;
      m_sample = NULL;
    }

  m_action = NULL;
  m_snapshot = NULL;
} // model::stop_action()

/*----------------------------------------------------------------------------*/
/**
 * \brief Reset the current action.
 * \param d The date at which the action is reset (new start).
 */
template<class Base>
void bear::engine::model<Base>::reset_action( universe::time_type d )
{
  CLAW_PRECOND(m_action != NULL);

  m_date = d;
  m_snapshot = m_action->get_snapshot_at(m_date);

  model_action::mark_iterator it;

  for ( it=m_action->mark_begin(); it!=m_action->mark_end(); ++it )
    if ( it->has_animation() )
      if ( it->get_animation()->is_finite() )
        {
          it->get_animation()->reset();
          it->get_animation()->next(d);
        }
} // model::reset_action()

/*----------------------------------------------------------------------------*/
/**
 * \brief Update the position of the sound emitted by the model.
 */
template<class Base>
void bear::engine::model<Base>::update_sound_position()
{
  if ( (m_sample != NULL) && (m_action!=NULL) )
    if ( !m_action->sound_is_global() )
      {
        audio::sound_effect effect( m_sample->get_effect() );
        effect.set_position( this->get_center_of_mass() );
        m_sample->set_effect( effect );
      }
} // model::update_sound_position()

/*----------------------------------------------------------------------------*/
/**
 * \brief Do the progression of the animations of the marks.
 * \param elapsed_time Elapsed time since the last call.
 */
template<class Base>
void bear::engine::model<Base>::progress_animations
( universe::time_type elapsed_time )
{
  // we must pay attention not to call next() several times on the same
  // animation, in the case where an animation is shared by several marks.
  std::set<model_animation> anim;
  model_action::mark_iterator it;
  
  for ( it=m_action->mark_begin(); it!=m_action->mark_end(); ++it )
    if ( it->has_animation() )
      anim.insert( it->get_animation() );

  for ( ; !anim.empty(); anim.erase(anim.begin()) )
    {
      model_animation a( *anim.begin() );
      a->next(elapsed_time);
    }
} // model::progress_animations()

/*----------------------------------------------------------------------------*/
/**
 * \brief Execute the current action.
 */
template<class Base>
void bear::engine::model<Base>::execute_snapshot()
{
  std::string s( m_snapshot->get_sound_name() );

  if ( this->get_level_globals().sound_exists(s) )
    if ( this->get_level_globals().new_sample(s) )
      {
        audio::sound_effect effect;
          
        if ( !m_snapshot->sound_is_global() )
          effect.set_position( this->get_center_of_mass() );
          
        effect.set_loops(1);
        this->get_level_globals().play_sound( s, effect );
      }
  
  update_bounding_box();
  execute_function( m_snapshot->get_function() );
} // model::execute_snapshot()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the bounding box of the item according to the new action.
 */
template<class Base>
void bear::engine::model<Base>::update_bounding_box()
{
  update_bounding_box_width();
  update_bounding_box_height();
} // model::update_bounding_box()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the width bounding box of the item according to the new action.
 */
template<class Base>
void bear::engine::model<Base>::update_bounding_box_width()
{
  universe::coordinate_type delta( m_snapshot->get_x_alignment_value() );
  universe::coordinate_type pos;
  model_snapshot::horizontal_alignment::value a
    ( m_snapshot->get_x_alignment() );

  if ( this->get_rendering_attributes().is_mirrored() )
    {
      if ( a == model_snapshot::horizontal_alignment::align_left )
        a = model_snapshot::horizontal_alignment::align_right;
      else if ( a == model_snapshot::horizontal_alignment::align_right )
        a = model_snapshot::horizontal_alignment::align_left;

      delta = -delta;
    }

  switch ( a )
    {
    case model_snapshot::horizontal_alignment::align_left:
      pos = this->get_left();
      this->set_width( m_snapshot->get_width() );
      this->set_left( pos + delta );
      break;
    case model_snapshot::horizontal_alignment::align_right:
      pos = this->get_right();
      this->set_width( m_snapshot->get_width() );
      this->set_right( pos + delta );
      break;
    case model_snapshot::horizontal_alignment::align_center:
      pos = this->get_horizontal_middle();
      this->set_width( m_snapshot->get_width() );
      this->set_horizontal_middle( pos + delta );
    }
} // model::update_bounding_box_width()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the height bounding box of the item according to the new action.
 */
template<class Base>
void bear::engine::model<Base>::update_bounding_box_height()
{
  universe::coordinate_type delta( m_snapshot->get_y_alignment_value() );
  universe::coordinate_type pos;
  model_snapshot::vertical_alignment::value a( m_snapshot->get_y_alignment() );

  if ( this->get_rendering_attributes().is_flipped() )
    {
      if ( a == model_snapshot::vertical_alignment::align_bottom )
        a = model_snapshot::vertical_alignment::align_top;
      else if ( a == model_snapshot::vertical_alignment::align_top )
        a = model_snapshot::vertical_alignment::align_bottom;

      delta = -delta;
    }

  switch ( a )
    {
    case model_snapshot::vertical_alignment::align_bottom:
      pos = this->get_bottom();
      this->set_height( m_snapshot->get_height() );
      this->set_bottom( pos + delta );
      break;
    case model_snapshot::vertical_alignment::align_top:
      pos = this->get_top();
      this->set_height( m_snapshot->get_height() );
      this->set_top( pos + delta );
      break;
    case model_snapshot::vertical_alignment::align_center:
      pos = this->get_vertical_middle();
      this->set_height( m_snapshot->get_height() );
      this->set_vertical_middle( pos + delta );
    }
} // model::update_bounding_box_height()

/*----------------------------------------------------------------------------*/
/**
 * \brief Clear the model. Delete the functions, the sample.
 */
template<class Base>
void bear::engine::model<Base>::clear()
{
  delete m_sample;
  m_sample = NULL;

  m_action = NULL;
  m_action_name = "";
  m_snapshot = NULL;
} // model::clear()
