/*
  Bear Engine - Editor library

  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file bf/code/item_instance.cpp
 * \brief Implementation of the bf::item_instance class.
 * \author Julien Jorge
 */
#include "bf/item_instance.hpp"

#include "bf/animation_file_xml_reader.hpp"
#include "bf/path_configuration.hpp"
#include "bf/wx_facilities.hpp"

#include <claw/assert.hpp>
#include <sstream>

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 * \param c The class I am an instance of.
 */
bf::item_instance::item_instance( item_class const* c )
  : m_class( c ), m_fixed( m_class->get_fixable() ),
    m_rendering_parameters(*this)
{
  m_rendering_parameters.init();
} // item_instance::item_instance()

/*----------------------------------------------------------------------------*/
/**
 * \brief Copy constructor.
 * \param that The instance to copy from.
 */
bf::item_instance::item_instance( const item_instance& that )
  : m_class(that.m_class), m_fixed( that.get_fixed() ),
    m_id(that.get_id()), m_rendering_parameters(*this)
{
  assign(that);
} // item_instance::item_instance()

/*----------------------------------------------------------------------------*/
/**
 * \brief Assignment.
 * \param that The instance to copy from.
 */
bf::item_instance& bf::item_instance::operator=( const item_instance& that )
{
  if ( this != &that )
    assign(that);

  return *this;
} // item_instance::operator=()

/*----------------------------------------------------------------------------*/
/**
 * \brief Copy the values of the fields of an other instance.
 * \param that The instance to copy from.
 */
void bf::item_instance::copy_fields_from( const item_instance& that )
{
  copy_fields_from_no_invalid_values( that );
  remove_invalid_values();

  m_rendering_parameters.reset();
} // item_instance::()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the rendering parameters of the item.
 */
const bf::item_rendering_parameters&
bf::item_instance::get_rendering_parameters() const
{
  return m_rendering_parameters;
} // item_instance::get_rendering_parameters()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the rendering parameters of the item.
 */
bf::item_rendering_parameters& bf::item_instance::get_rendering_parameters()
{
  return m_rendering_parameters;
} // item_instance::get_rendering_parameters()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the class I am an instance of.
 */
const bf::item_class& bf::item_instance::get_class() const
{
  return *m_class;
} // item_instance::get_class()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the pointer to the class I am an instance of.
 */
bf::item_class const* bf::item_instance::get_class_ptr() const
{
  return m_class;
} // item_instance::get_class_ptr()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the name of the class of the item.
 * \param c The new class of the item.
 * \pre c!=NULL
 */
void bf::item_instance::set_class( item_class const* c )
{
  CLAW_PRECOND( c!=NULL );

  m_class = c;
  m_fixed = m_class->get_fixable() && m_fixed;
  m_rendering_parameters.set_is_valid(true);

  remove_invalid_values();

  m_rendering_parameters.reset();
} // item_instance::set_class()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the the identifier of this item.
 */
const std::string& bf::item_instance::get_id() const
{
  return m_id;
} // item_instance::get_id()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the the identifier of this item.
 * \param id The new identifier of this item.
 */
void bf::item_instance::set_id( const std::string& id )
{
  m_id = id;
} // item_instance::set_id()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell if a field has a value.
 * \param f The field to check.
 */
bool bf::item_instance::has_value( const type_field& f ) const
{
  bool result = false;

  if ( f.is_list() )
    switch ( f.get_field_type() )
      {
      case type_field::integer_field_type:
        result = m_int_list.find(f.get_name()) != m_int_list.end();
        break;
      case type_field::u_integer_field_type:
        result = m_u_int_list.find(f.get_name()) != m_u_int_list.end();
        break;
      case type_field::real_field_type:
        result = m_real_list.find(f.get_name()) != m_real_list.end();
        break;
      case type_field::boolean_field_type:
        result = m_bool_list.find(f.get_name()) != m_bool_list.end();
        break;
      case type_field::string_field_type:
        result = m_string_list.find(f.get_name()) != m_string_list.end();
        break;
      case type_field::sprite_field_type:
        result = m_sprite_list.find(f.get_name()) != m_sprite_list.end();
        break;
      case type_field::animation_field_type:
        result = m_animation_list.find(f.get_name()) != m_animation_list.end();
        break;
      case type_field::item_reference_field_type:
        result = ( m_item_reference_list.find(f.get_name())
                   != m_item_reference_list.end() );
        break;
      case type_field::font_field_type:
        result = m_font_list.find(f.get_name()) != m_font_list.end();
        break;
      case type_field::sample_field_type:
        result = m_sample_list.find(f.get_name()) != m_sample_list.end();
        break;
      }
  else
    switch ( f.get_field_type() )
      {
      case type_field::integer_field_type:
        result = m_int.find(f.get_name()) != m_int.end();
        break;
      case type_field::u_integer_field_type:
        result = m_u_int.find(f.get_name()) != m_u_int.end();
        break;
      case type_field::real_field_type:
        result = m_real.find(f.get_name()) != m_real.end();
        break;
      case type_field::boolean_field_type:
        result = m_bool.find(f.get_name()) != m_bool.end();
        break;
      case type_field::string_field_type:
        result = m_string.find(f.get_name()) != m_string.end();
        break;
      case type_field::sprite_field_type:
        result = m_sprite.find(f.get_name()) != m_sprite.end();
        break;
      case type_field::animation_field_type:
        result = m_animation.find(f.get_name()) != m_animation.end();
        break;
      case type_field::item_reference_field_type:
        result = m_item_reference.find(f.get_name()) != m_item_reference.end();
        break;
      case type_field::font_field_type:
        result = m_font.find(f.get_name()) != m_font.end();
        break;
      case type_field::sample_field_type:
        result = m_sample.find(f.get_name()) != m_sample.end();
        break;
      }

  return result;
} // item_instance::has_value()

/*----------------------------------------------------------------------------*/
/**
 * \brief Delete the value of a field.
 * \param f The field to delete.
 */
void bf::item_instance::delete_value( const type_field& f )
{
  if ( has_value(f) )
    {
      if ( f.is_list() )
        switch ( f.get_field_type() )
          {
          case type_field::integer_field_type:
            m_int_list.erase( f.get_name() );
            break;
          case type_field::u_integer_field_type:
            m_u_int_list.erase( f.get_name() );
            break;
          case type_field::real_field_type:
            m_real_list.erase( f.get_name() );
            break;
          case type_field::boolean_field_type:
            m_bool_list.erase( f.get_name() );
            break;
          case type_field::string_field_type:
            m_string_list.erase( f.get_name() );
            break;
          case type_field::sprite_field_type:
            m_sprite_list.erase( f.get_name() );
            break;
          case type_field::animation_field_type:
            m_animation_list.erase( f.get_name() );
            break;
          case type_field::item_reference_field_type:
            m_item_reference_list.erase( f.get_name() );
            break;
	  case type_field::font_field_type:
            m_font_list.erase( f.get_name() );
            break;
	  case type_field::sample_field_type:
            m_sample_list.erase( f.get_name() );
            break;
          }
      else
        switch ( f.get_field_type() )
          {
          case type_field::integer_field_type:
            m_int.erase( f.get_name() );
            break;
          case type_field::u_integer_field_type:
            m_u_int.erase( f.get_name() );
            break;
          case type_field::real_field_type:
            m_real.erase( f.get_name() );
            break;
          case type_field::boolean_field_type:
            m_bool.erase( f.get_name() );
            break;
          case type_field::string_field_type:
            m_string.erase( f.get_name() );
            break;
          case type_field::sprite_field_type:
            m_sprite.erase( f.get_name() );
            break;
          case type_field::animation_field_type:
            m_animation.erase( f.get_name() );
            break;
          case type_field::item_reference_field_type:
            m_item_reference.erase( f.get_name() );
            break;
	  case type_field::font_field_type:
            m_font.erase( f.get_name() );
            break;
	  case type_field::sample_field_type:
            m_sample.erase( f.get_name() );
            break;
          }
    }

  m_rendering_parameters.field_changed( f.get_name() );
} // item_instance::delete_value()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell if the item is fixed.
 */
bool bf::item_instance::get_fixed() const
{
  return m_fixed && m_class->get_fixable();
} // item_instance::get_fixed()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell that this item is fixed.
 * \param b Fixed or not ?
 */
void bf::item_instance::set_fixed( bool b )
{
  m_fixed = b;
} // item_instance::set_fixed()

/*----------------------------------------------------------------------------*/
/**
 * \brief Compile the fields of the item.
 * \param f The file in which we compile.
 * \param id_to_int Integer values associated to the items id, to use for
 *        references.
 */
void bf::item_instance::compile
( compiled_file& f, const std::map<std::string, unsigned int>& id_to_int ) const
{
  f << get_fixed();

  std::list<std::string> fields;

  for( sort_fields(fields); !fields.empty(); fields.pop_front() )
    compile_field( f, m_class->get_field(fields.front()), id_to_int );
} // item_instance::compile()

/*----------------------------------------------------------------------------*/
/**
 * \brief Test if the item has a good initialization.
 * Return "true" if there are no problem.
 * Test item references and required fields.
 * \param error_msg The string of error message.
 * \param map_id The map containing item identifiants.
 */
bool bf::item_instance::check
( std::string& error_msg, const std::map<std::string,item_instance*>& map_id)
{
  bool result = check_required_fields(error_msg);
  
  if ( result ) 
    result = check_mass_for_fixed_item(error_msg);
  
  if ( result ) 
    result = check_id_required(error_msg, map_id);

  if ( result ) 
    {
      m_rendering_parameters.clear_error();
      m_rendering_parameters.set_is_valid(true);
    }
  else
    {
      m_rendering_parameters.set_error(error_msg);
      m_rendering_parameters.set_is_valid(false);
    }  

  return result;
} // item_instance::check()

/*----------------------------------------------------------------------------*/
/**
 * \brief Test if each required fields are setted.
 * Return "true" if there are no problem.
 * \param error_msg The string of error message.
 */
bool bf::item_instance::check_mass_for_fixed_item(std::string& error_msg) const
{
  bool result = true;

  if ( m_fixed ) 
    {
      type_field field("base_item.mass",type_field::real_field_type);
      if ( has_value(field) )
	{
	  result = false;
	  error_msg = "Error item " + m_class->get_class_name() +
	    " : for fixed item, the field 'base_item.mass' must be" +
	    " equal to infinity";
	}
    }

  return result;
} // item_instance::check_mass_for_fixed_item()

/*----------------------------------------------------------------------------*/
/**
 * \brief Test if each required identifier exists.
 * Return "true" if there are no problem.
 * \param error_msg The string of error message.
 * \param map_id The map containing item identifiants.
 */
bool bf::item_instance::check_id_required
(std::string& error_msg, 
 const std::map<std::string,item_instance*>& map_id) const
{
  bool result = true;

  std::list<std::string> fields;
  std::list<std::string>::const_iterator it;

  m_class->get_field_names_in_hierarchy(fields);

  for ( it=fields.begin(); it!=fields.end(); ++it )
    {
      const type_field& f = m_class->get_field(*it);
      if( f.get_field_type() == type_field::item_reference_field_type )
        {
	  if( has_value(f) )
	    {
	      if( f.is_list() )
		{
		  std::list<item_reference_type>::const_iterator bit =
		    m_item_reference_list.find(f.get_name())->second.begin();
		  const std::list<item_reference_type>::const_iterator eit =
		    m_item_reference_list.find(f.get_name())->second.end();
		  
		  for ( ; bit!=eit; ++bit)
		    if ( map_id.find(bit->get_value()) == map_id.end() )
		      {
			result = false;
			error_msg = "Error item " + m_class->get_class_name() +
			  " : field '" + *it + 
			  "'. The identifier '" + bit->get_value() 
			  + "' does not exist";
		      }	     
		}
	      else
		{
		  std::string id;
		  id = m_item_reference.find(f.get_name())->second.get_value();
		  
		  if ( map_id.find(id) == map_id.end() ) 
		    {
		      result = false;
		      error_msg = "Error item " + m_class->get_class_name() +
			" : field '" + *it + 
			"'. The identifier '" + id + "' does not exist";
		    }
		}
	    }
	}
    }

  return result;
} // item_instance::check_id_required()

/*----------------------------------------------------------------------------*/
/**
 * \brief Test if each required fields are setted.
 * Return "true" if there are no problem.
 * \param error_msg The string of error message.
 */
bool bf::item_instance::check_required_fields(std::string& error_msg) const
{
  bool result = true;

  std::list<std::string> fields;
  std::list<std::string>::const_iterator it;

  m_class->get_field_names_in_hierarchy(fields);

  for ( it=fields.begin(); it!=fields.end(); ++it )
    {
      const type_field& f = m_class->get_field(*it);
      if( f.get_required() )
        if( !has_value(f) )
          {
            result = false;
            error_msg = "Error item " + m_class->get_class_name() +
              " : field '" + *it + "' is required";
          }
    }

  return result;
} // item_instance::check_required_fields()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the name of the fields, ordered so that fields depending on other
 *        fields are after them in the list.
 * \param fields (out) The name of the fields.
 */
void bf::item_instance::sort_fields( std::list<std::string>& fields ) const
{
  std::set<std::string> all_fields;

  copy_field_names( m_int, all_fields );
  copy_field_names( m_u_int, all_fields );
  copy_field_names( m_real, all_fields );
  copy_field_names( m_bool, all_fields );
  copy_field_names( m_string, all_fields );
  copy_field_names( m_sprite, all_fields );
  copy_field_names( m_animation, all_fields );
  copy_field_names( m_item_reference, all_fields );
  copy_field_names( m_font, all_fields );
  copy_field_names( m_sample, all_fields );
  copy_field_names( m_int_list, all_fields );
  copy_field_names( m_u_int_list, all_fields );
  copy_field_names( m_real_list, all_fields );
  copy_field_names( m_bool_list, all_fields );
  copy_field_names( m_string_list, all_fields );
  copy_field_names( m_sprite_list, all_fields );
  copy_field_names( m_animation_list, all_fields );
  copy_field_names( m_item_reference_list, all_fields );
  copy_field_names( m_font_list, all_fields );
  copy_field_names( m_sample_list, all_fields );

  while ( !all_fields.empty() )
    {
      const std::string field_name( *all_fields.begin() );
      insert_field( field_name, fields, all_fields );
    }
} // item_instance::sort_fields()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the name of the fields, ordered so that fields depending on other
 *        fields are after them in the list.
 * \param field_name The name of the field to insert.
 * \param fields (out) The name of the fields.
 * \param all_fields (in/out) All the remaining fields to insert.
 */
void bf::item_instance::insert_field
( const std::string& field_name, std::list<std::string>& fields,
  std::set<std::string>& all_fields ) const
{
  if ( all_fields.find(field_name) != all_fields.end() )
    {
      all_fields.erase(field_name);

      std::set<std::string>::const_iterator it;

      for ( it=m_class->get_field(field_name).get_preceding().begin();
            it!=m_class->get_field(field_name).get_preceding().end(); ++it )
        insert_field(*it, fields, all_fields);

      fields.push_back( field_name );
    }
} // item_instance::insert_field()

/*----------------------------------------------------------------------------*/
/**
 * \brief Compile a field.
 * \param f The file in which we compile.
 * \param field The field to save.
 * \param id_to_int Integer values associated to the items id, to use for
 *        references.
 */
void bf::item_instance::compile_field
( compiled_file& f, const type_field& field,
  const std::map<std::string, unsigned int>& id_to_int ) const
{
  CLAW_PRECOND( has_value(field) );

  if ( field.is_list() )
    f << bear::level_code_value::field_list;

  f << get_code_value(field) << field.get_name();

  if ( field.is_list() )
    compile_field_list(f, field, id_to_int);
  else
    compile_field_single(f, field, id_to_int);
} // item_instance::compile_field()

/*----------------------------------------------------------------------------*/
/**
 * \brief Compile a field made of a single values.
 * \param f The file in which we compile.
 * \param field The field to save.
 * \param id_to_int Integer values associated to the items id, to use for
 *        references.
 */
void bf::item_instance::compile_field_single
( compiled_file& f, const type_field& field,
  const std::map<std::string, unsigned int>& id_to_int ) const
{
  CLAW_PRECOND( has_value(field) );
  const std::string& field_name = field.get_name();

  switch ( field.get_field_type() )
    {
    case type_field::integer_field_type:
      m_int.find(field_name)->second.compile(f);
      break;
    case type_field::u_integer_field_type:
      m_u_int.find(field_name)->second.compile(f);
      break;
    case type_field::real_field_type:
      m_real.find(field_name)->second.compile(f);
      break;
    case type_field::boolean_field_type:
      m_bool.find(field_name)->second.compile(f);
      break;
    case type_field::string_field_type:
      m_string.find(field_name)->second.compile(f);
      break;
    case type_field::sprite_field_type:
      m_sprite.find(field_name)->second.compile(f);
      break;
    case type_field::animation_field_type:
      m_animation.find(field_name)->second.compile(f);
      break;
    case type_field::item_reference_field_type:
      f << id_to_int.find
        ( m_item_reference.find(field_name)->second.get_value())->second;
      break;
    case type_field::font_field_type:
      m_font.find(field_name)->second.compile(f);
      break;
    case type_field::sample_field_type:
      m_sample.find(field_name)->second.compile(f);
      break;
    }
} // item_instance::compile_field_single()

/*----------------------------------------------------------------------------*/
/**
 * \brief Compile a field made of a list of values.
 * \param f The file in which we compile.
 * \param field The field to save.
 * \param id_to_int Integer values associated to the items id, to use for
 *        references.
 */
void bf::item_instance::compile_field_list
( compiled_file& f, const type_field& field,
  const std::map<std::string, unsigned int>& id_to_int ) const
{
  CLAW_PRECOND( has_value(field) );
  const std::string& field_name = field.get_name();

  switch ( field.get_field_type() )
    {
    case type_field::integer_field_type:
      compile_list( f, m_int_list.find(field_name)->second );
      break;
    case type_field::u_integer_field_type:
      compile_list( f, m_u_int_list.find(field_name)->second );
      break;
    case type_field::real_field_type:
      compile_list( f, m_real_list.find(field_name)->second );
      break;
    case type_field::boolean_field_type:
      compile_list( f, m_bool_list.find(field_name)->second );
      break;
    case type_field::string_field_type:
      compile_list( f, m_string_list.find(field_name)->second );
      break;
    case type_field::sprite_field_type:
      compile_list( f, m_sprite_list.find(field_name)->second );
      break;
    case type_field::animation_field_type:
      compile_list( f, m_animation_list.find(field_name)->second );
      break;
    case type_field::item_reference_field_type:
      {
        f << m_item_reference_list.find(field_name)->second.size();

        std::list<item_reference_type>::const_iterator it =
          m_item_reference_list.find(field_name)->second.begin();
        const std::list<item_reference_type>::const_iterator eit =
          m_item_reference_list.find(field_name)->second.end();

        for ( ; it!=eit; ++it)
          f << id_to_int.find(it->get_value())->second;

        break;
      }
    case type_field::font_field_type:
      compile_list( f, m_font_list.find(field_name)->second );
      break;
    case type_field::sample_field_type:
      compile_list( f, m_sample_list.find(field_name)->second );
      break;
    }
} // item_instance::compile_field_list()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the compiler code value for type of field.
 * \param field The field forwhich we want the value.
 */
bear::level_code_value::value_type
bf::item_instance::get_code_value( const type_field& field ) const
{
  bear::level_code_value::value_type result;

  switch ( field.get_field_type() )
    {
    case type_field::integer_field_type:
      result = bear::level_code_value::field_int;
      break;
    case type_field::u_integer_field_type:
      result = bear::level_code_value::field_u_int;
      break;
    case type_field::real_field_type:
      result = bear::level_code_value::field_real;
      break;
    case type_field::boolean_field_type:
      result = bear::level_code_value::field_bool;
      break;
    case type_field::string_field_type:
      result = bear::level_code_value::field_string;
      break;
    case type_field::sprite_field_type:
      result = bear::level_code_value::field_sprite;
      break;
    case type_field::animation_field_type:
      result = bear::level_code_value::field_animation;
      break;
    case type_field::item_reference_field_type:
      result = bear::level_code_value::field_item;
      break;
    case type_field::font_field_type:
      result = bear::level_code_value::field_string;
      break;
    case type_field::sample_field_type:
      result = bear::level_code_value::field_string;
      break;
    default:
      {
        CLAW_ASSERT( false, "Invalid field type." );
      }
    }

  return result;
} // item_instance::get_code_value()

/*----------------------------------------------------------------------------*/
/**
 * \brief Assign the value from an other item.
 * \param that The instance to copy from.
 */
void bf::item_instance::assign( const item_instance& that )
{
  m_class = that.m_class;
  m_fixed = that.get_fixed();
  m_id = that.get_id();

  copy_fields_from_no_invalid_values( that );

  m_rendering_parameters.reset();
} // item_instance::assign()

/*----------------------------------------------------------------------------*/
/**
 * \brief Copy the values of the fields of an other instance without removing
 *        the invalid values.
 * \param that The instance to copy the fields from.
 */
void bf::item_instance::copy_fields_from_no_invalid_values
( const item_instance& that )
{
  m_int = that.m_int;
  m_int_list = that.m_int_list;
  m_u_int = that.m_u_int;
  m_u_int_list = that.m_u_int_list;
  m_real = that.m_real;
  m_real_list = that.m_real_list;
  m_bool = that.m_bool;
  m_bool_list = that.m_bool_list;
  m_string = that.m_string;
  m_string_list = that.m_string_list;
  m_sprite = that.m_sprite;
  m_sprite_list = that.m_sprite_list;
  m_animation = that.m_animation;
  m_animation_list = that.m_animation_list;
  m_item_reference = that.m_item_reference;
  m_item_reference_list = that.m_item_reference_list;
  m_font = that.m_font;
  m_font_list = that.m_font_list;
  m_sample = that.m_sample;
  m_sample_list = that.m_sample_list;
} // item_instance::copy_fields_from_no_invalid_values()

/*----------------------------------------------------------------------------*/
/**
 * \brief Remove the values set for some fields not present in the item class.
 */
void bf::item_instance::remove_invalid_values()
{
  remove_invalid_values( m_int, m_int_list, type_field::integer_field_type );
  remove_invalid_values
    ( m_u_int, m_u_int_list, type_field::u_integer_field_type );
  remove_invalid_values( m_real, m_real_list, type_field::real_field_type );
  remove_invalid_values( m_bool, m_bool_list, type_field::boolean_field_type );
  remove_invalid_values
    ( m_string, m_string_list, type_field::string_field_type );
  remove_invalid_values
    ( m_sprite, m_sprite_list, type_field::sprite_field_type );
  remove_invalid_values
    ( m_animation, m_animation_list, type_field::animation_field_type );
  remove_invalid_values
    ( m_item_reference, m_item_reference_list,
      type_field::item_reference_field_type );
  remove_invalid_values
    ( m_font, m_font_list, type_field::font_field_type );
  remove_invalid_values
    ( m_sample, m_sample_list, type_field::sample_field_type );
} // item_instance::set_class()
