/*
    Bear Engine - Editor library

    Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    contact: plee-the-bear@gamned.org

    Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file bf/item_instance.hpp
 * \brief An instance of an item class.
 * \author Julien Jorge
 */
#ifndef __BF_ITEM_INSTANCE_HPP__
#define __BF_ITEM_INSTANCE_HPP__

#include <map>
#include <string>
#include <set>
#include <list>
#include <vector>

#include <claw/assert.hpp>

#include "bf/animation.hpp"
#include "bf/custom_type.hpp"
#include "bf/item_rendering_parameters.hpp"
#include "bf/type_field.hpp"
#include "bf/libeditor_export.hpp"

#include "level_code_value.hpp"

/**
 * \brief Declare the member variables and accessors for a field of a given
 *        type.
 * \param type The type of the field.
 * \param name Root name of the variables created.
 *
 * This macro declares two private variables named m_(name) and m_(name)_list,
 * of type std::map<std::string, (type)> and std::map<std::string,
 * std::list<(type)> >.
 *
 * Methods to access these variables are also created:
 *
 *  void set_value( const std::string&, const (type)& )
 *  void set_value( const std::string&, const std::list<(type)>& )
 *  void get_value( const std::string&, (type)& ) const
 *  void get_value( const std::string&, std::list<(type)>& ) const
 *  void get_value( const type_field&, (type)& ) const
 *  void get_value( const type_field&, std::list<(type)>& ) const
 */
#define FIELD_TYPE( type, name )                                        \
  public:                                                               \
  void set_value( const std::string& field_name, const type& v )        \
  {                                                                     \
    m_ ## name[field_name] = v;                                         \
    m_rendering_parameters.field_changed(field_name);                   \
  }                                                                     \
                                                                        \
  void                                                                  \
  set_value( const std::string& field_name, const std::list<type>& v )  \
  {                                                                     \
    m_ ## name ## _list[field_name] = v;                                \
    m_rendering_parameters.field_changed(field_name);                   \
  }                                                                     \
                                                                        \
  void get_value( const std::string& field_name, type& v ) const        \
  {                                                                     \
    CLAW_PRECOND( m_ ## name.find(field_name) != m_ ## name.end() );    \
    v = m_ ## name.find(field_name)->second;                            \
  }                                                                     \
                                                                        \
  void                                                                  \
  get_value( const std::string& field_name, std::list<type>& v ) const  \
  {                                                                     \
    CLAW_PRECOND( m_ ## name ## _list.find(field_name)                  \
                  != m_ ## name ## _list.end() );                       \
    v = m_ ## name ## _list.find(field_name)->second;                   \
  }                                                                     \
                                                                        \
  void get_value( const type_field& f, type& v ) const                  \
  {                                                                     \
    get_value( f.get_name(), v );                                       \
  }                                                                     \
                                                                        \
  void get_value( const type_field& f, std::list<type>& v ) const       \
  {                                                                     \
    get_value( f.get_name(), v );                                       \
  }                                                                     \
                                                                        \
  private:                                                              \
  std::map<std::string, type> m_ ## name;                               \
  std::map< std::string, std::list<type> > m_ ## name ## _list

namespace bf
{
  class item_class;

  /**
   * \brief An instance of an item class.
   * \author Julien Jorge
   */
  class BEAR_EDITOR_EXPORT item_instance
  {
  public:
    item_instance( item_class const* c );
    item_instance( const item_instance& that );

    item_instance& operator=( const item_instance& that );

    void copy_fields_from( const item_instance& that );

    const item_rendering_parameters& get_rendering_parameters() const;
    item_rendering_parameters& get_rendering_parameters();

    const item_class& get_class() const;
    item_class const* get_class_ptr() const;
    void set_class( item_class const* c );

    const std::string& get_id() const;
    void set_id( const std::string& id );

    bool has_value( const type_field& f ) const;
    void delete_value( const type_field& f );

    bool get_fixed() const;
    void set_fixed( bool b );

    void compile( compiled_file& f,
                  const std::map<std::string, unsigned int>& id_to_int ) const;

    bool check
    ( std::string& error_msg,
      const std::map<std::string,item_instance*>& map_id );

  private:
    bool check_mass_for_fixed_item(std::string& error_msg) const;
    bool check_id_required
    (std::string& error_msg,
     const std::map<std::string,item_instance*>& map_id) const;
    bool check_required_fields(std::string& error_msg) const;

    void sort_fields( std::list<std::string>& fields ) const;
    void insert_field
    ( const std::string& field_name, std::list<std::string>& fields,
      std::set<std::string>& all_fields ) const;

    void compile_field
    ( compiled_file& f, const type_field& field,
      const std::map<std::string, unsigned int>& id_to_int ) const;
    void compile_field_single
    ( compiled_file& f, const type_field& field,
      const std::map<std::string, unsigned int>& id_to_int ) const;
    void compile_field_list
    ( compiled_file& f, const type_field& field,
      const std::map<std::string, unsigned int>& id_to_int ) const;

    bear::level_code_value::value_type
    get_code_value( const type_field& field ) const;

    void assign( const item_instance& that );
    void copy_fields_from_no_invalid_values( const item_instance& that );
    void remove_invalid_values();

    template<typename MapType>
    void
    copy_field_names( const MapType& m, std::set<std::string>& fields ) const;

    template<typename T>
    void compile_list( compiled_file& f, const std::list<T>& v ) const;

    template<typename T>
    void remove_invalid_values
    ( std::map<std::string, T>& m, std::map< std::string, std::list<T> >& ml,
      type_field::field_type t ) const;

  private:
    /** \brief The item class I am an instance of. */
    item_class const* m_class;

    /** \brief Tell if the item is fixed. */
    bool m_fixed;

    /** \brief The identifier of this item, for references with other items. */
    std::string m_id;

    /** \brief Rendering parameters of the instance. */
    item_rendering_parameters m_rendering_parameters;

    FIELD_TYPE( integer_type, int );
    FIELD_TYPE( u_integer_type, u_int );
    FIELD_TYPE( real_type, real );
    FIELD_TYPE( bool_type, bool );
    FIELD_TYPE( string_type, string );
    FIELD_TYPE( sprite, sprite );
    FIELD_TYPE( animation_file_type, animation );
    FIELD_TYPE( item_reference_type, item_reference );
    FIELD_TYPE( font_file_type, font );
    FIELD_TYPE( sample_file_type, sample );
  }; // class item_instance
} // namespace bf

#include "bf/impl/item_instance.tpp"

#endif // __BF_ITEM_INSTANCE_HPP__
