/*
  Plee the Bear

  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [PTB] in the subject of your mails.
*/
/**
 * \file level_ending_effect.cpp
 * \brief Implementation of the ptb::level_ending_effect class.
 * \author Julien Jorge
 */
#include "ptb/transition_effect/level_ending_effect.hpp"

#include "engine/game.hpp"
#include "engine/level.hpp"
#include "visual/scene_sprite.hpp"
#include "visual/scene_line.hpp"
#include "visual/scene_writing.hpp"

#include "ptb/defines.hpp"
#include "ptb/level_variables.hpp"
#include "ptb/item/timer_kill_players.hpp"
#include "ptb/item/plee/plee.hpp"

#include <libintl.h>

/*----------------------------------------------------------------------------*/
const bear::visual::coordinate_type
ptb::level_ending_effect::score_line::s_shadow_delta(2);
const double ptb::level_ending_effect::score_line::s_scale_factor(0.5);

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 * \param f The font to use for the texts.
 * \param text A short description of the reason we're giving the points for.
 * \param points The initial score.
 */
ptb::level_ending_effect::score_line::score_line
( const bear::visual::font& f, const std::string& text, unsigned int points )
  : m_font(f), m_label(m_font, text), m_points(points), m_y(0)
{
  // force creating the writing
  decrease_score(0);
} // level_ending_effect::score_line::score_line()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the scene elements of the score.
 * \param e (out) The scene element of the score.
 * \param left The minimum x-position of the elements.
 * \param right The maximum x-position of the elements.
 */
void ptb::level_ending_effect::score_line::render
( scene_element_list& e, bear::visual::coordinate_type left,
  bear::visual::coordinate_type right ) const
{
  bear::visual::scene_writing label
    (left+s_shadow_delta, m_y+s_shadow_delta, m_label);

  label.set_scale_factor(s_scale_factor, s_scale_factor);
  label.get_rendering_attributes().set_intensity(0, 0, 0);

  e.push_back( label );

  std::vector<bear::visual::position_type> p(2);
  p[0].set(left, m_y + get_height());
  p[1].set(right, m_y + get_height());

  e.push_back( bear::visual::scene_line( 0, 0, PTB_ORANGE_PIXEL, p, 1) );
  e.push_back
    ( bear::visual::scene_line
      ( s_shadow_delta, s_shadow_delta, claw::graphic::black_pixel, p, 1) );

  label.set_position(left-s_shadow_delta, m_y-s_shadow_delta);
  label.get_rendering_attributes().set_intensity(1, 1, 1);

  e.push_back( label );

  bear::visual::coordinate_type x
    (right - m_points_text.get_width() * s_scale_factor);
  bear::visual::scene_writing points
    (x+s_shadow_delta, m_y+s_shadow_delta, m_points_text);

  points.set_scale_factor(s_scale_factor, s_scale_factor);
  points.get_rendering_attributes().set_intensity(0, 0, 0);

  e.push_back( points );

  points.set_position(x-s_shadow_delta, m_y-s_shadow_delta);
  points.get_rendering_attributes().set_intensity(1, 1, 1);

  e.push_back( points );
} // level_ending_effect::score_line::render()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the score.
 */
unsigned int ptb::level_ending_effect::score_line::get_score() const
{
  return m_points;
} // level_ending_effect::score_line::get_score()

/*----------------------------------------------------------------------------*/
/**
 * \brief Decrease the score.
 * \param delta The value of the decrement.
 * \return How much the score was decreased (ie. min(delta, get_score())).
 */
unsigned int
ptb::level_ending_effect::score_line::decrease_score( unsigned int delta )
{
  unsigned int s;

  if (m_points <= delta)
    s = m_points;
  else
    s = delta;

  m_points -= s;

  std::ostringstream oss;
  oss << m_points;

  m_points_text.create(m_font, oss.str());

  return s;
} // level_ending_effect::score_line::decrease_score()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the height of the line.
 */
bear::visual::coordinate_type
ptb::level_ending_effect::score_line::get_height() const
{
  return m_label.get_height() * 0.5;
} // level_ending_effect::score_line::get_height()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the y-position of the line.
 */
bear::visual::coordinate_type
ptb::level_ending_effect::score_line::get_y_position() const
{
  return m_y;
} // level_ending_effect::score_line::get_y_position()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the y-position of the line.
 * \param y The position.
 */
void ptb::level_ending_effect::score_line::set_y_position
( bear::visual::coordinate_type y )
{
  m_y = y;
} // level_ending_effect::score_line::set_y_position()




/*----------------------------------------------------------------------------*/
const unsigned int ptb::level_ending_effect::s_points_per_second(1000);
const bear::visual::coordinate_type
ptb::level_ending_effect::s_screen_margin(100);
const bear::visual::coordinate_type ptb::level_ending_effect::s_margin(10);
const double ptb::level_ending_effect::s_score_line_speed(50);

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 */
ptb::level_ending_effect::level_ending_effect()
  : m_speed_factor(1), m_next_tick(0.1)
{

} // level_ending_effect::level_ending_effect()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell if the effect is finished.
 */
bool ptb::level_ending_effect::is_finished() const
{
  return false;
} // level_ending_effect::is_finished()

/*----------------------------------------------------------------------------*/
/**
 * \brief Initialise the effect.
 */
void ptb::level_ending_effect::build()
{
  bear::visual::font f
    ( get_level_globals().get_font("font/level_name-42x50.fnt") );

  m_points.push_back
    ( score_line
      ( f, gettext("Honey pots"),
        5000 * level_variables::get_honeypots_found( get_level() ) ) );

  if ( level_variables::get_azelnut( get_level() ) )
    m_points.push_back( score_line( f, gettext("Azelnut"), 5000 ) );

  if ( level_variables::owl_is_met( get_level() ) )
    m_points.push_back
      ( score_line( f, gettext("Meet the owl"), 10000 ) );

  if ( level_variables::get_killed_enemies( get_level() )
       == level_variables::get_enemy_count( get_level() ) )
    m_points.push_back( score_line( f, gettext("Carnage"), 20000 ) );

  if ( !level_variables::get_time_exceeded( get_level() ) )
    {
      timer_kill_players::msg_get_value msg;
      bear::engine::level_globals& glob = get_level_globals();

      if ( glob.send_message( "level_timer", msg ) )
        {
          if (msg.value >= level_variables::get_time_record( get_level() ) )
            m_points.push_back
              ( score_line( f, gettext("Rocket bonus"), 10000 ) );

          m_points.push_back
              ( score_line( f, gettext("Time bonus"), msg.value * 10 ) );
        }
    }

  std::list<score_line>::iterator it;
  bear::visual::coordinate_type y(100);

  for (it=m_points.begin(); it!=m_points.end(); ++it)
    {
      it->set_y_position(y);
      y += it->get_height() + s_margin;
    }
} // level_ending_effect::build()

/*----------------------------------------------------------------------------*/
/**
 * \brief Adjust the components of the effect.
 * \param elapsed_time Elapsed time since the last call.
 */
bear::universe::time_type
ptb::level_ending_effect::progress( bear::universe::time_type elapsed_time )
{
  std::list<score_line>::iterator it;
  // is it the time to start the next level?
  bool start_level(true);
  bool stop(false);
  bear::visual::coordinate_type y(s_screen_margin);

  for (it=m_points.begin(); !stop && (it!=m_points.end()); ++it)
    if ( it->get_score() != 0 )
      {
        start_level = false;

        const unsigned int s =
          it->decrease_score
          (s_points_per_second * m_speed_factor * elapsed_time);

        game_variables::add_score(1, s);
        game_variables::add_score(2, s);

        if ( it->get_y_position() > y )
          it->set_y_position
            ( std::max
              ((double)y,
               it->get_y_position() - s_score_line_speed * elapsed_time) );

        y += it->get_height() + s_margin;
      }

  if ( m_next_tick > elapsed_time )
    m_next_tick -= elapsed_time;
  else
    {
      get_level_globals().play_sound("sound/tick.ogg");

      if ( m_speed_factor > 1 )
        m_next_tick = 0.05;
      else
        m_next_tick = 0.1;
    }

  if (start_level)
    bear::engine::game::get_instance().set_waiting_level
      ( PTB_LOADING_LEVEL_NAME );

  m_speed_factor = 1;

  return 0;
} // level_ending_effect::progress()

/*----------------------------------------------------------------------------*/
/**
 * \brief Render the components of the effect.
 * \param e (out) The scene elements.
 */
void ptb::level_ending_effect::render( scene_element_list& e ) const
{
  bear::visual::coordinate_type left(s_screen_margin);
  bear::visual::coordinate_type right(get_layer().get_size().x -  left);
  bool stop(false);
  const bear::visual::coordinate_type bottom
    (get_layer().get_size().y - s_screen_margin);
  std::list<score_line>::const_iterator it;

  for (it=m_points.begin(); !stop && (it!=m_points.end()); ++it)
    if ( it->get_score() != 0 )
      {
        if ( it->get_y_position() >= bottom )
          stop = true;
        else
          it->render(e, left, right);
      }
          
} // level_ending_effect::render()

/*----------------------------------------------------------------------------*/
/**
 * \brief Inform the effect that a key had been pressed.
 * \param key The value of the pressed key.
 * \remark This effect pretends to treat all keys, so we can maintain the pause
 *         mode.
 */
bool ptb::level_ending_effect::key_pressed( const bear::input::key_info& key )
{
  // prevent pausing the game
  return true;
} // level_ending_effect::key_pressed()

/*----------------------------------------------------------------------------*/
/**
 * \brief Inform the effect that a key is maintained.
 * \param key The value of the pressed key.
 */
bool
ptb::level_ending_effect::key_maintained( const bear::input::key_info& key )
{
  m_speed_factor = 10;

  return true;
} // level_ending_effect::key_maintained()

/*----------------------------------------------------------------------------*/
/**
 * \brief Inform the effect that a joystick button had been pressed.
 * \param button The value of the pressed button.
 * \param joy_index The index of the joystick.
 * \remark This effect pretends to treat all keys, so we can maintain the pause
 *         mode.
 */
bool ptb::level_ending_effect::button_pressed
( bear::input::joystick::joy_code button, unsigned int joy_index )
{
  // prevent pausing the game
  return true;
} // level_ending_effect::button_pressed()

/*----------------------------------------------------------------------------*/
/**
 * \brief Inform the effect that a joystick button is maintained.
 * \param button The value of the pressed button.
 * \param joy_index The index of the joystick.
 */
bool ptb::level_ending_effect::button_maintained
( bear::input::joystick::joy_code button, unsigned int joy_index )
{
  m_speed_factor = 10;

  return true;
} // level_ending_effect::button_maintained()
